/*
*    Yerase's TNEF Stream Reader Library
*    Copyright (C) 2003  Randall E. Hand
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*    You can contact me at randall.hand@gmail.com for questions or assistance
*/
#ifndef MAPITAGS_H
#define MAPITAGS_H

#define MAPIMACRO(x) {x, #x}
typedef struct {
  unsigned int id;
  char name[128];
} MAPIPropertyTagList;


MAPIPropertyTagList MPList[] = {
  MAPIMACRO(PR_ACKNOWLEDGEMENT_MODE),
  MAPIMACRO(PR_ALTERNATE_RECIPIENT_ALLOWED),
  MAPIMACRO(PR_AUTHORIZING_USERS),
  MAPIMACRO(PR_AUTO_FORWARD_COMMENT),
  MAPIMACRO(PR_AUTO_FORWARDED),
  MAPIMACRO(PR_CONTENT_CONFIDENTIALITY_ALGORITHM_ID),
  MAPIMACRO(PR_CONTENT_CORRELATOR),
  MAPIMACRO(PR_CONTENT_IDENTIFIER),
  MAPIMACRO(PR_CONTENT_LENGTH),
  MAPIMACRO(PR_CONTENT_RETURN_REQUESTED),
  MAPIMACRO(PR_CONVERSATION_KEY),
  MAPIMACRO(PR_CONVERSION_EITS),
  MAPIMACRO(PR_CONVERSION_WITH_LOSS_PROHIBITED),
  MAPIMACRO(PR_CONVERTED_EITS),
  MAPIMACRO(PR_DEFERRED_DELIVERY_TIME),
  MAPIMACRO(PR_DELIVER_TIME),
  MAPIMACRO(PR_DISCARD_REASON),
  MAPIMACRO(PR_DISCLOSURE_OF_RECIPIENTS),
  MAPIMACRO(PR_DL_EXPANSION_HISTORY),
  MAPIMACRO(PR_DL_EXPANSION_PROHIBITED),
  MAPIMACRO(PR_EXPIRY_TIME),
  MAPIMACRO(PR_IMPLICIT_CONVERSION_PROHIBITED),
  MAPIMACRO(PR_IMPORTANCE),
  MAPIMACRO(PR_IPM_ID),
  MAPIMACRO(PR_LATEST_DELIVERY_TIME),
  MAPIMACRO(PR_MESSAGE_CLASS),
  MAPIMACRO(PR_MESSAGE_DELIVERY_ID),
  MAPIMACRO(PR_MESSAGE_SECURITY_LABEL),
  MAPIMACRO(PR_OBSOLETED_IPMS),
  MAPIMACRO(PR_ORIGINALLY_INTENDED_RECIPIENT_NAME),
  MAPIMACRO(PR_ORIGINAL_EITS),
  MAPIMACRO(PR_ORIGINATOR_CERTIFICATE),
  MAPIMACRO(PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED),
  MAPIMACRO(PR_ORIGINATOR_RETURN_ADDRESS),
  MAPIMACRO(PR_PARENT_KEY),
  MAPIMACRO(PR_PRIORITY),
  MAPIMACRO(PR_ORIGIN_CHECK),
  MAPIMACRO(PR_PROOF_OF_SUBMISSION_REQUESTED),
  MAPIMACRO(PR_READ_RECEIPT_REQUESTED),
  MAPIMACRO(PR_RECEIPT_TIME),
  MAPIMACRO(PR_RECIPIENT_REASSIGNMENT_PROHIBITED),
  MAPIMACRO(PR_REDIRECTION_HISTORY),
  MAPIMACRO(PR_RELATED_IPMS),
  MAPIMACRO(PR_ORIGINAL_SENSITIVITY),
  MAPIMACRO(PR_LANGUAGES),
  MAPIMACRO(PR_REPLY_TIME),
  MAPIMACRO(PR_REPORT_TAG),
  MAPIMACRO(PR_REPORT_TIME),
  MAPIMACRO(PR_RETURNED_IPM),
  MAPIMACRO(PR_SECURITY),
  MAPIMACRO(PR_INCOMPLETE_COPY),
  MAPIMACRO(PR_SENSITIVITY),
  MAPIMACRO(PR_SUBJECT),
  MAPIMACRO(PR_SUBJECT_IPM),
  MAPIMACRO(PR_CLIENT_SUBMIT_TIME),
  MAPIMACRO(PR_REPORT_NAME),
  MAPIMACRO(PR_SENT_REPRESENTING_SEARCH_KEY),
  MAPIMACRO(PR_X400_CONTENT_TYPE),
  MAPIMACRO(PR_SUBJECT_PREFIX),
  MAPIMACRO(PR_NON_RECEIPT_REASON),
  MAPIMACRO(PR_RECEIVED_BY_ENTRYID),
  MAPIMACRO(PR_RECEIVED_BY_NAME),
  MAPIMACRO(PR_SENT_REPRESENTING_ENTRYID),
  MAPIMACRO(PR_SENT_REPRESENTING_NAME),
  MAPIMACRO(PR_RCVD_REPRESENTING_ENTRYID),
  MAPIMACRO(PR_RCVD_REPRESENTING_NAME),
  MAPIMACRO(PR_REPORT_ENTRYID),
  MAPIMACRO(PR_READ_RECEIPT_ENTRYID),
  MAPIMACRO(PR_MESSAGE_SUBMISSION_ID),
  MAPIMACRO(PR_PROVIDER_SUBMIT_TIME),
  MAPIMACRO(PR_ORIGINAL_SUBJECT),
  MAPIMACRO(PR_DISC_VAL),
  MAPIMACRO(PR_ORIG_MESSAGE_CLASS),
  MAPIMACRO(PR_ORIGINAL_AUTHOR_ENTRYID),
  MAPIMACRO(PR_ORIGINAL_AUTHOR_NAME),
  MAPIMACRO(PR_ORIGINAL_SUBMIT_TIME),
  MAPIMACRO(PR_REPLY_RECIPIENT_ENTRIES),
  MAPIMACRO(PR_REPLY_RECIPIENT_NAMES),
  MAPIMACRO(PR_RECEIVED_BY_SEARCH_KEY),
  MAPIMACRO(PR_RCVD_REPRESENTING_SEARCH_KEY),
  MAPIMACRO(PR_READ_RECEIPT_SEARCH_KEY),
  MAPIMACRO(PR_REPORT_SEARCH_KEY),
  MAPIMACRO(PR_ORIGINAL_DELIVERY_TIME),
  MAPIMACRO(PR_ORIGINAL_AUTHOR_SEARCH_KEY),
  MAPIMACRO(PR_MESSAGE_TO_ME),
  MAPIMACRO(PR_MESSAGE_CC_ME),
  MAPIMACRO(PR_MESSAGE_RECIP_ME),
  MAPIMACRO(PR_ORIGINAL_SENDER_NAME),
  MAPIMACRO(PR_ORIGINAL_SENDER_ENTRYID),
  MAPIMACRO(PR_ORIGINAL_SENDER_SEARCH_KEY),
  MAPIMACRO(PR_ORIGINAL_SENT_REPRESENTING_NAME),
  MAPIMACRO(PR_ORIGINAL_SENT_REPRESENTING_ENTRYID),
  MAPIMACRO(PR_ORIGINAL_SENT_REPRESENTING_SEARCH_KEY),
  MAPIMACRO(PR_START_DATE),
  MAPIMACRO(PR_END_DATE),
  MAPIMACRO(PR_OWNER_APPT_ID),
  MAPIMACRO(PR_RESPONSE_REQUESTED),
  MAPIMACRO(PR_SENT_REPRESENTING_ADDRTYPE),
  MAPIMACRO(PR_SENT_REPRESENTING_EMAIL_ADDRESS),
  MAPIMACRO(PR_ORIGINAL_SENDER_ADDRTYPE),
  MAPIMACRO(PR_ORIGINAL_SENDER_EMAIL_ADDRESS),
  MAPIMACRO(PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE),
  MAPIMACRO(PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS),
  MAPIMACRO(PR_CONVERSATION_TOPIC),
  MAPIMACRO(PR_CONVERSATION_INDEX),
  MAPIMACRO(PR_ORIGINAL_DISPLAY_BCC),
  MAPIMACRO(PR_ORIGINAL_DISPLAY_CC),
  MAPIMACRO(PR_ORIGINAL_DISPLAY_TO),
  MAPIMACRO(PR_RECEIVED_BY_ADDRTYPE),
  MAPIMACRO(PR_RECEIVED_BY_EMAIL_ADDRESS),
  MAPIMACRO(PR_RCVD_REPRESENTING_ADDRTYPE),
  MAPIMACRO(PR_RCVD_REPRESENTING_EMAIL_ADDRESS),
  MAPIMACRO(PR_ORIGINAL_AUTHOR_ADDRTYPE),
  MAPIMACRO(PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS),
  MAPIMACRO(PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE),
  MAPIMACRO(PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS),
  MAPIMACRO(PR_TRANSPORT_MESSAGE_HEADERS),
  MAPIMACRO(PR_DELEGATION),
  MAPIMACRO(PR_TNEF_CORRELATION_KEY),
  MAPIMACRO(PR_CONTENT_INTEGRITY_CHECK),
  MAPIMACRO(PR_EXPLICIT_CONVERSION),
  MAPIMACRO(PR_IPM_RETURN_REQUESTED),
  MAPIMACRO(PR_MESSAGE_TOKEN),
  MAPIMACRO(PR_NDR_REASON_CODE),
  MAPIMACRO(PR_NDR_DIAG_CODE),
  MAPIMACRO(PR_NON_RECEIPT_NOTIFICATION_REQUESTED),
  MAPIMACRO(PR_DELIVERY_POINT),
  MAPIMACRO(PR_ORIGINATOR_NON_DELIVERY_REPORT_REQUESTED),
  MAPIMACRO(PR_ORIGINATOR_REQUESTED_ALTERNATE_RECIPIENT),
  MAPIMACRO(PR_PHYSICAL_DELIVERY_BUREAU_FAX_DELIVERY),
  MAPIMACRO(PR_PHYSICAL_DELIVERY_MODE),
  MAPIMACRO(PR_PHYSICAL_DELIVERY_REPORT_REQUEST),
  MAPIMACRO(PR_PHYSICAL_FORWARDING_ADDRESS),
  MAPIMACRO(PR_PHYSICAL_FORWARDING_ADDRESS_REQUESTED),
  MAPIMACRO(PR_PHYSICAL_FORWARDING_PROHIBITED),
  MAPIMACRO(PR_PHYSICAL_RENDITION_ATTRIBUTES),
  MAPIMACRO(PR_PROOF_OF_DELIVERY),
  MAPIMACRO(PR_PROOF_OF_DELIVERY_REQUESTED),
  MAPIMACRO(PR_RECIPIENT_CERTIFICATE),
  MAPIMACRO(PR_RECIPIENT_NUMBER_FOR_ADVICE),
  MAPIMACRO(PR_RECIPIENT_TYPE),
  MAPIMACRO(PR_REGISTERED_MAIL_TYPE),
  MAPIMACRO(PR_REPLY_REQUESTED),
  MAPIMACRO(PR_REQUESTED_DELIVERY_METHOD),
  MAPIMACRO(PR_SENDER_ENTRYID),
  MAPIMACRO(PR_SENDER_NAME),
  MAPIMACRO(PR_SUPPLEMENTARY_INFO),
  MAPIMACRO(PR_TYPE_OF_MTS_USER),
  MAPIMACRO(PR_SENDER_SEARCH_KEY),
  MAPIMACRO(PR_SENDER_ADDRTYPE),
  MAPIMACRO(PR_SENDER_EMAIL_ADDRESS),
  MAPIMACRO(PR_CURRENT_VERSION),
  MAPIMACRO(PR_DELETE_AFTER_SUBMIT),
  MAPIMACRO(PR_DISPLAY_BCC),
  MAPIMACRO(PR_DISPLAY_CC),
  MAPIMACRO(PR_DISPLAY_TO),
  MAPIMACRO(PR_PARENT_DISPLAY),
  MAPIMACRO(PR_MESSAGE_DELIVERY_TIME),
  MAPIMACRO(PR_MESSAGE_FLAGS),
  MAPIMACRO(PR_MESSAGE_SIZE),
  MAPIMACRO(PR_PARENT_ENTRYID),
  MAPIMACRO(PR_SENTMAIL_ENTRYID),
  MAPIMACRO(PR_CORRELATE),
  MAPIMACRO(PR_CORRELATE_MTSID),
  MAPIMACRO(PR_DISCRETE_VALUES),
  MAPIMACRO(PR_RESPONSIBILITY),
  MAPIMACRO(PR_SPOOLER_STATUS),
  MAPIMACRO(PR_TRANSPORT_STATUS),
  MAPIMACRO(PR_MESSAGE_RECIPIENTS),
  MAPIMACRO(PR_MESSAGE_ATTACHMENTS),
  MAPIMACRO(PR_SUBMIT_FLAGS),
  MAPIMACRO(PR_RECIPIENT_STATUS),
  MAPIMACRO(PR_TRANSPORT_KEY),
  MAPIMACRO(PR_MSG_STATUS),
  MAPIMACRO(PR_MESSAGE_DOWNLOAD_TIME),
  MAPIMACRO(PR_CREATION_VERSION),
  MAPIMACRO(PR_MODIFY_VERSION),
  MAPIMACRO(PR_HASATTACH),
  MAPIMACRO(PR_BODY_CRC),
  MAPIMACRO(PR_NORMALIZED_SUBJECT),
  MAPIMACRO(PR_RTF_IN_SYNC),
  MAPIMACRO(PR_ATTACH_SIZE),
  MAPIMACRO(PR_ATTACH_NUM),
  MAPIMACRO(PR_PREPROCESS),
  MAPIMACRO(PR_ORIGINATING_MTA_CERTIFICATE),
  MAPIMACRO(PR_PROOF_OF_SUBMISSION),
  MAPIMACRO(PR_ENTRYID),
  MAPIMACRO(PR_OBJECT_TYPE),
  MAPIMACRO(PR_ICON),
  MAPIMACRO(PR_MINI_ICON),
  MAPIMACRO(PR_STORE_ENTRYID),
  MAPIMACRO(PR_STORE_RECORD_KEY),
  MAPIMACRO(PR_RECORD_KEY),
  MAPIMACRO(PR_MAPPING_SIGNATURE),
  MAPIMACRO(PR_ACCESS_LEVEL),
  MAPIMACRO(PR_INSTANCE_KEY),
  MAPIMACRO(PR_ROW_TYPE),
  MAPIMACRO(PR_ACCESS),
  MAPIMACRO(PR_BODY),
  MAPIMACRO(PR_REPORT_TEXT),
  MAPIMACRO(PR_ORIGINATOR_AND_DL_EXPANSION_HISTORY),
  MAPIMACRO(PR_REPORTING_DL_NAME),
  MAPIMACRO(PR_REPORTING_MTA_CERTIFICATE),
  MAPIMACRO(PR_RTF_SYNC_BODY_CRC),
  MAPIMACRO(PR_RTF_SYNC_BODY_COUNT),
  MAPIMACRO(PR_RTF_SYNC_BODY_TAG),
  MAPIMACRO(PR_RTF_COMPRESSED),
  MAPIMACRO(PR_RTF_SYNC_PREFIX_COUNT),
  MAPIMACRO(PR_RTF_SYNC_TRAILING_COUNT),
  MAPIMACRO(PR_ORIGINALLY_INTENDED_RECIP_ENTRYID),
  MAPIMACRO(PR_BODY_HTML),
  MAPIMACRO(PR_INTERNET_MESSAGE_ID),
  MAPIMACRO(PR_ICON_INDEX),
  MAPIMACRO(PR_ROWID),
  MAPIMACRO(PR_DISPLAY_NAME),
  MAPIMACRO(PR_ADDRTYPE),
  MAPIMACRO(PR_EMAIL_ADDRESS),
  MAPIMACRO(PR_COMMENT),
  MAPIMACRO(PR_DEPTH),
  MAPIMACRO(PR_PROVIDER_DISPLAY),
  MAPIMACRO(PR_CREATION_TIME),
  MAPIMACRO(PR_LAST_MODIFICATION_TIME),
  MAPIMACRO(PR_RESOURCE_FLAGS),
  MAPIMACRO(PR_PROVIDER_DLL_NAME),
  MAPIMACRO(PR_SEARCH_KEY),
  MAPIMACRO(PR_PROVIDER_UID),
  MAPIMACRO(PR_PROVIDER_ORDINAL),
  MAPIMACRO(PR_FORM_VERSION),
  MAPIMACRO(PR_FORM_CLSID),
  MAPIMACRO(PR_FORM_CONTACT_NAME),
  MAPIMACRO(PR_FORM_CATEGORY),
  MAPIMACRO(PR_FORM_CATEGORY_SUB),
  MAPIMACRO(PR_FORM_HOST_MAP),
  MAPIMACRO(PR_FORM_HIDDEN),
  MAPIMACRO(PR_FORM_DESIGNER_NAME),
  MAPIMACRO(PR_FORM_DESIGNER_GUID),
  MAPIMACRO(PR_FORM_MESSAGE_BEHAVIOR),
  MAPIMACRO(PR_DEFAULT_STORE),
  MAPIMACRO(PR_STORE_SUPPORT_MASK),
  MAPIMACRO(PR_STORE_STATE),
  MAPIMACRO(PR_IPM_SUBTREE_SEARCH_KEY),
  MAPIMACRO(PR_IPM_OUTBOX_SEARCH_KEY),
  MAPIMACRO(PR_IPM_WASTEBASKET_SEARCH_KEY),
  MAPIMACRO(PR_IPM_SENTMAIL_SEARCH_KEY),
  MAPIMACRO(PR_MDB_PROVIDER),
  MAPIMACRO(PR_RECEIVE_FOLDER_SETTINGS),
  MAPIMACRO(PR_VALID_FOLDER_MASK),
  MAPIMACRO(PR_IPM_SUBTREE_ENTRYID),
  MAPIMACRO(PR_IPM_OUTBOX_ENTRYID),
  MAPIMACRO(PR_IPM_WASTEBASKET_ENTRYID),
  MAPIMACRO(PR_IPM_SENTMAIL_ENTRYID),
  MAPIMACRO(PR_VIEWS_ENTRYID),
  MAPIMACRO(PR_COMMON_VIEWS_ENTRYID),
  MAPIMACRO(PR_FINDER_ENTRYID),
  MAPIMACRO(PR_CONTAINER_FLAGS),
  MAPIMACRO(PR_FOLDER_TYPE),
  MAPIMACRO(PR_CONTENT_COUNT),
  MAPIMACRO(PR_CONTENT_UNREAD),
  MAPIMACRO(PR_CREATE_TEMPLATES),
  MAPIMACRO(PR_DETAILS_TABLE),
  MAPIMACRO(PR_SEARCH),
  MAPIMACRO(PR_SELECTABLE),
  MAPIMACRO(PR_SUBFOLDERS),
  MAPIMACRO(PR_STATUS),
  MAPIMACRO(PR_ANR),
  MAPIMACRO(PR_CONTENTS_SORT_ORDER),
  MAPIMACRO(PR_CONTAINER_HIERARCHY),
  MAPIMACRO(PR_CONTAINER_CONTENTS),
  MAPIMACRO(PR_FOLDER_ASSOCIATED_CONTENTS),
  MAPIMACRO(PR_DEF_CREATE_DL),
  MAPIMACRO(PR_DEF_CREATE_MAILUSER),
  MAPIMACRO(PR_CONTAINER_CLASS),
  MAPIMACRO(PR_CONTAINER_MODIFY_VERSION),
  MAPIMACRO(PR_AB_PROVIDER_ID),
  MAPIMACRO(PR_DEFAULT_VIEW_ENTRYID),
  MAPIMACRO(PR_ASSOC_CONTENT_COUNT),
  MAPIMACRO(PR_ATTACHMENT_X400_PARAMETERS),
  MAPIMACRO(PR_ATTACH_DATA_OBJ),
  MAPIMACRO(PR_ATTACH_ENCODING),
  MAPIMACRO(PR_ATTACH_EXTENSION),
  MAPIMACRO(PR_ATTACH_FILENAME),
  MAPIMACRO(PR_ATTACH_METHOD),
  MAPIMACRO(PR_ATTACH_LONG_FILENAME),
  MAPIMACRO(PR_ATTACH_PATHNAME),
  MAPIMACRO(PR_ATTACH_RENDERING),
  MAPIMACRO(PR_ATTACH_TAG),
  MAPIMACRO(PR_RENDERING_POSITION),
  MAPIMACRO(PR_ATTACH_TRANSPORT_NAME),
  MAPIMACRO(PR_ATTACH_LONG_PATHNAME),
  MAPIMACRO(PR_ATTACH_MIME_TAG),
  MAPIMACRO(PR_ATTACH_ADDITIONAL_INFO),
  MAPIMACRO(PR_ATTACH_MIME_SEQUENCE),
  MAPIMACRO(PR_ATTACH_CONTENT_ID),
  MAPIMACRO(PR_ATTACH_CONTENT_LOCATION),
  MAPIMACRO(PR_ATTACH_FLAGS),
  MAPIMACRO(PR_DISPLAY_TYPE),
  MAPIMACRO(PR_TEMPLATEID),
  MAPIMACRO(PR_PRIMARY_CAPABILITY),
  MAPIMACRO(PR_7BIT_DISPLAY_NAME),
  MAPIMACRO(PR_ACCOUNT),
  MAPIMACRO(PR_ALTERNATE_RECIPIENT),
  MAPIMACRO(PR_CALLBACK_TELEPHONE_NUMBER),
  MAPIMACRO(PR_CONVERSION_PROHIBITED),
  MAPIMACRO(PR_DISCLOSE_RECIPIENTS),
  MAPIMACRO(PR_GENERATION),
  MAPIMACRO(PR_GIVEN_NAME),
  MAPIMACRO(PR_GOVERNMENT_ID_NUMBER),
  MAPIMACRO(PR_BUSINESS_TELEPHONE_NUMBER),
  MAPIMACRO(PR_HOME_TELEPHONE_NUMBER),
  MAPIMACRO(PR_INITIALS),
  MAPIMACRO(PR_KEYWORD),
  MAPIMACRO(PR_LANGUAGE),
  MAPIMACRO(PR_LOCATION),
  MAPIMACRO(PR_MAIL_PERMISSION),
  MAPIMACRO(PR_MHS_COMMON_NAME),
  MAPIMACRO(PR_ORGANIZATIONAL_ID_NUMBER),
  MAPIMACRO(PR_SURNAME),
  MAPIMACRO(PR_ORIGINAL_ENTRYID),
  MAPIMACRO(PR_ORIGINAL_DISPLAY_NAME),
  MAPIMACRO(PR_ORIGINAL_SEARCH_KEY),
  MAPIMACRO(PR_POSTAL_ADDRESS),
  MAPIMACRO(PR_COMPANY_NAME),
  MAPIMACRO(PR_TITLE),
  MAPIMACRO(PR_DEPARTMENT_NAME),
  MAPIMACRO(PR_OFFICE_LOCATION),
  MAPIMACRO(PR_PRIMARY_TELEPHONE_NUMBER),
  MAPIMACRO(PR_BUSINESS2_TELEPHONE_NUMBER),
  MAPIMACRO(PR_MOBILE_TELEPHONE_NUMBER),
  MAPIMACRO(PR_RADIO_TELEPHONE_NUMBER),
  MAPIMACRO(PR_CAR_TELEPHONE_NUMBER),
  MAPIMACRO(PR_OTHER_TELEPHONE_NUMBER),
  MAPIMACRO(PR_TRANSMITABLE_DISPLAY_NAME),
  MAPIMACRO(PR_PAGER_TELEPHONE_NUMBER),
  MAPIMACRO(PR_USER_CERTIFICATE),
  MAPIMACRO(PR_PRIMARY_FAX_NUMBER),
  MAPIMACRO(PR_BUSINESS_FAX_NUMBER),
  MAPIMACRO(PR_HOME_FAX_NUMBER),
  MAPIMACRO(PR_COUNTRY),
  MAPIMACRO(PR_LOCALITY),
  MAPIMACRO(PR_STATE_OR_PROVINCE),
  MAPIMACRO(PR_STREET_ADDRESS),
  MAPIMACRO(PR_POSTAL_CODE),
  MAPIMACRO(PR_POST_OFFICE_BOX),
  MAPIMACRO(PR_TELEX_NUMBER),
  MAPIMACRO(PR_ISDN_NUMBER),
  MAPIMACRO(PR_ASSISTANT_TELEPHONE_NUMBER),
  MAPIMACRO(PR_HOME2_TELEPHONE_NUMBER),
  MAPIMACRO(PR_ASSISTANT),
  MAPIMACRO(PR_SEND_RICH_INFO),
  MAPIMACRO(PR_WEDDING_ANNIVERSARY),
  MAPIMACRO(PR_BIRTHDAY),
  MAPIMACRO(PR_HOBBIES),
  MAPIMACRO(PR_MIDDLE_NAME),
  MAPIMACRO(PR_DISPLAY_NAME_PREFIX),
  MAPIMACRO(PR_PROFESSION),
  MAPIMACRO(PR_PREFERRED_BY_NAME),
  MAPIMACRO(PR_SPOUSE_NAME),
  MAPIMACRO(PR_COMPUTER_NETWORK_NAME),
  MAPIMACRO(PR_CUSTOMER_ID),
  MAPIMACRO(PR_TTYTDD_PHONE_NUMBER),
  MAPIMACRO(PR_FTP_SITE),
  MAPIMACRO(PR_GENDER),
  MAPIMACRO(PR_MANAGER_NAME),
  MAPIMACRO(PR_NICKNAME),
  MAPIMACRO(PR_PERSONAL_HOME_PAGE),
  MAPIMACRO(PR_BUSINESS_HOME_PAGE),
  MAPIMACRO(PR_CONTACT_VERSION),
  MAPIMACRO(PR_CONTACT_ENTRYIDS),
  MAPIMACRO(PR_CONTACT_ADDRTYPES),
  MAPIMACRO(PR_CONTACT_DEFAULT_ADDRESS_INDEX),
  MAPIMACRO(PR_CONTACT_EMAIL_ADDRESSES),
  MAPIMACRO(PR_COMPANY_MAIN_PHONE_NUMBER),
  MAPIMACRO(PR_CHILDRENS_NAMES),
  MAPIMACRO(PR_HOME_ADDRESS_CITY),
  MAPIMACRO(PR_HOME_ADDRESS_COUNTRY),
  MAPIMACRO(PR_HOME_ADDRESS_POSTAL_CODE),
  MAPIMACRO(PR_HOME_ADDRESS_STATE_OR_PROVINCE),
  MAPIMACRO(PR_HOME_ADDRESS_STREET),
  MAPIMACRO(PR_HOME_ADDRESS_POST_OFFICE_BOX),
  MAPIMACRO(PR_OTHER_ADDRESS_CITY),
  MAPIMACRO(PR_OTHER_ADDRESS_COUNTRY),
  MAPIMACRO(PR_OTHER_ADDRESS_POSTAL_CODE),
  MAPIMACRO(PR_OTHER_ADDRESS_STATE_OR_PROVINCE),
  MAPIMACRO(PR_OTHER_ADDRESS_STREET),
  MAPIMACRO(PR_OTHER_ADDRESS_POST_OFFICE_BOX),
  MAPIMACRO(PR_STORE_PROVIDERS),
  MAPIMACRO(PR_AB_PROVIDERS),
  MAPIMACRO(PR_TRANSPORT_PROVIDERS),
  MAPIMACRO(PR_DEFAULT_PROFILE),
  MAPIMACRO(PR_AB_SEARCH_PATH),
  MAPIMACRO(PR_AB_DEFAULT_DIR),
  MAPIMACRO(PR_AB_DEFAULT_PAB),
  MAPIMACRO(PR_FILTERING_HOOKS),
  MAPIMACRO(PR_SERVICE_NAME),
  MAPIMACRO(PR_SERVICE_DLL_NAME),
  MAPIMACRO(PR_SERVICE_ENTRY_NAME),
  MAPIMACRO(PR_SERVICE_UID),
  MAPIMACRO(PR_SERVICE_EXTRA_UIDS),
  MAPIMACRO(PR_SERVICES),
  MAPIMACRO(PR_SERVICE_SUPPORT_FILES),
  MAPIMACRO(PR_SERVICE_DELETE_FILES),
  MAPIMACRO(PR_AB_SEARCH_PATH_UPDATE),
  MAPIMACRO(PR_PROFILE_NAME),
  MAPIMACRO(PR_IDENTITY_DISPLAY),
  MAPIMACRO(PR_IDENTITY_ENTRYID),
  MAPIMACRO(PR_RESOURCE_METHODS),
  MAPIMACRO(PR_RESOURCE_TYPE),
  MAPIMACRO(PR_STATUS_CODE),
  MAPIMACRO(PR_IDENTITY_SEARCH_KEY),
  MAPIMACRO(PR_OWN_STORE_ENTRYID),
  MAPIMACRO(PR_RESOURCE_PATH),
  MAPIMACRO(PR_STATUS_STRING),
  MAPIMACRO(PR_X400_DEFERRED_DELIVERY_CANCEL),
  MAPIMACRO(PR_HEADER_FOLDER_ENTRYID),
  MAPIMACRO(PR_REMOTE_PROGRESS),
  MAPIMACRO(PR_REMOTE_PROGRESS_TEXT),
  MAPIMACRO(PR_REMOTE_VALIDATE_OK),
  MAPIMACRO(PR_CONTROL_FLAGS),
  MAPIMACRO(PR_CONTROL_STRUCTURE),
  MAPIMACRO(PR_CONTROL_TYPE),
  MAPIMACRO(PR_DELTAX),
  MAPIMACRO(PR_DELTAY),
  MAPIMACRO(PR_XPOS),
  MAPIMACRO(PR_YPOS),
  MAPIMACRO(PR_CONTROL_ID),
  MAPIMACRO(PR_INITIAL_DETAILS_PANE),
  MAPIMACRO(PR_INTERNET_CPID),
  MAPIMACRO(PR_MESSAGE_LOCALE_ID),
  MAPIMACRO(PR_CREATOR_NAME),
  MAPIMACRO(PR_CREATOR_ENTRYID),
  MAPIMACRO(PR_LAST_MODIFIER_NAME),
  MAPIMACRO(PR_MESSAGE_CODEPAGE),
  MAPIMACRO(PR_INETMAIL_OVERRIDE_FORMAT),
  MAPIMACRO(PR_MSG_EDITOR_FORMAT),
  MAPIMACRO(PR_SENDER_SMTP_ADDRESS),
  MAPIMACRO(PR_SENT_REPRESENTING_SMTP_ADDRESS),
};

#endif  /* MAPITAGS_H */
